package com.squareup.sqldelight.core.views

import com.google.common.truth.Truth.assertThat
import com.squareup.sqldelight.core.compiler.SqlDelightCompiler
import com.squareup.sqldelight.test.util.FixtureCompiler
import org.junit.Rule
import org.junit.Test
import org.junit.rules.TemporaryFolder
import java.io.File

class InterfaceGeneration {
  @get:Rule val temporaryFolder = TemporaryFolder()

  @Test fun onlyTableType() {
    checkFixtureCompiles("only-table-type")
  }

  @Test fun requiresAdapter() {
    checkFixtureCompiles("requires-adapter")
  }

  @Test fun `view with exposed booleans through union`() {
    val result = FixtureCompiler.compileSql("""
      |CREATE TABLE test (
      |  val INTEGER AS Boolean NOT NULL
      |);
      |
      |CREATE VIEW someView AS
      |SELECT val, val
      |FROM test
      |UNION
      |SELECT 0, 0
      |FROM test;
      |""".trimMargin(), temporaryFolder)

    assertThat(result.errors).isEmpty()
    val generatedInterface = result.compilerOutput.get(
        File(result.outputDirectory, "com/example/SomeView.kt")
    )
    assertThat(generatedInterface).isNotNull()
    assertThat(generatedInterface.toString()).isEqualTo("""
      |package com.example
      |
      |import kotlin.Boolean
      |import kotlin.String
      |
      |interface SomeView {
      |    val val_: Boolean
      |
      |    val val__: Boolean
      |
      |    data class Impl(override val val_: Boolean, override val val__: Boolean) : SomeView {
      |        override fun toString(): String = ""${'"'}
      |        |SomeView.Impl [
      |        |  val_: ${"$"}val_
      |        |  val__: ${"$"}val__
      |        |]
      |        ""${'"'}.trimMargin()
      |    }
      |}
      |""".trimMargin())
  }

  @Test fun `view with exposed booleans through union of separate tables`() {
    val result = FixtureCompiler.compileSql("""
      |CREATE TABLE test (
      |  val INTEGER AS Boolean NOT NULL
      |);
      |
      |CREATE TABLE another_test (
      |  val INTEGER AS Boolean NOT NULL
      |);
      |
      |CREATE VIEW someView AS
      |SELECT val, val
      |FROM test
      |UNION
      |SELECT val, val
      |FROM another_test;
      |""".trimMargin(), temporaryFolder)

    assertThat(result.errors).isEmpty()
    val generatedInterface = result.compilerOutput.get(
        File(result.outputDirectory, "com/example/SomeView.kt")
    )
    assertThat(generatedInterface).isNotNull()
    assertThat(generatedInterface.toString()).isEqualTo("""
      |package com.example
      |
      |import kotlin.Boolean
      |import kotlin.String
      |
      |interface SomeView {
      |    val val_: Boolean
      |
      |    val val__: Boolean
      |
      |    data class Impl(override val val_: Boolean, override val val__: Boolean) : SomeView {
      |        override fun toString(): String = ""${'"'}
      |        |SomeView.Impl [
      |        |  val_: ${"$"}val_
      |        |  val__: ${"$"}val__
      |        |]
      |        ""${'"'}.trimMargin()
      |    }
      |}
      |""".trimMargin())
  }

  private fun checkFixtureCompiles(fixtureRoot: String) {
    val result = FixtureCompiler.compileFixture(
        "src/test/view-interface-fixtures/$fixtureRoot",
        SqlDelightCompiler::writeViewInterfaces,
        false)
    for ((expectedFile, actualOutput) in result.compilerOutput) {
      assertThat(expectedFile.exists()).named("No file with name $expectedFile").isTrue()
      assertThat(expectedFile.readText()).named(expectedFile.name).isEqualTo(
          actualOutput.toString())
    }
  }
}
